/*
** Useful pins: 
** 		PA0-PA4 lcd comm
** 		PA5, stanga , PA6 shoot , PA7 dreapta
** 
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "lcd.h"

static unsigned char x_ship = 40; /* human coord */
static unsigned char x_proj, y_proj = 41; /* munition coordi */

static unsigned char x_enemy = 41, y_enemy = 5; /* enemy coordinates */

static char x_e_step;
static unsigned char x_e_counter, y_e_counter;

static unsigned char enemymap[3][9]; /* 0 = mort, 1 = enemyA, 2 = enemyB */

static unsigned char enemies_alive;



char battleship[5][5] = {{0,0,1,0,0},
						  {0,0,1,0,0},
						  {0,0,1,0,0},
						  {1,1,1,1,1},
						  {1,1,1,1,1}};

char projectile[5][5] = {{0,0,0,0,0},
						  {0,0,1,0,0},
						  {0,1,1,1,0},
						  {0,0,1,0,0},
						  {0,0,1,0,0}};
						  					  
char enemyA[5][5] = 	 {{0,0,1,0,0},
						  {0,0,1,0,0},
						  {0,1,1,1,0},
						  {0,1,1,1,0},
						  {1,1,1,1,1}};
						  
						  
char enemyB[5][5] = 	 {{0,1,1,1,0},
						  {0,1,1,1,0},
						  {1,1,1,1,1},
						  {1,1,1,1,1},
						  {1,0,0,0,1}};

void end_lightshow();
void lightshow(char x);


void drawBattleship(unsigned char x, unsigned char y)
{
	char i, j;
	
	
	lcd_goto_xy_exact(0,0);
	for(i = 0; i < 5; i++)
		for(j = 0; j < 5; j++)
			if(battleship[i][j] == 1)
				lcd_setpixel(x + j,y + i);
	
}

void drawProjectile(unsigned char x, unsigned char y)
{
	char i, j;
	
	if (y_proj <= 40)
	{
		lcd_goto_xy_exact(0,0);
		for(i = 0; i < 5; i++)
			for(j = 0; j < 5; j++)
				if(projectile[i][j] == 1)
					lcd_setpixel(x + j,y + i);
	}
}


void move_ship()
{
	if (x_ship > 0)
		if (PINC & (1 << PC1))
			x_ship--;
			
	if (x_ship < 80)
		if (PINC & (1 << PC3))
			x_ship++;	
	
}

void launch_projectile()
{
	if (y_proj > 40)
		if (!(PINC & (1 << PC2)))
		{
			x_proj = x_ship;
			y_proj = 40;
		}
}

void move_projectile()
{
	if (y_proj <= 40)
	y_proj--;
}

void initButtons()
{
	DDRC &= ~(1 << PC1);  
	PORTC |= (1 << PC1);  
	
	DDRC &= ~(1 << PC2);  
	PORTC |= (1 << PC2);  
	
	DDRC &= ~(1 << PC3);  
	PORTC |= (1 << PC3);

	DDRC &= ~(1 << PC4);  
	PORTC |= (1 << PC4); 
}

void initEnemies()
{
	int i, j;
	
	x_enemy = 41;
	y_enemy = 5;
	x_e_counter = 0;
	x_e_step = 1;
	y_e_counter = 0;
	
	for(i = 0; i < 3; i++)
		for(j = 0; j < 9; j++)
			enemymap[i][j] = 1;
	
	enemies_alive = 27;
}

void drawEnemies()
{
	int i, j, x, y, k, l;
	
	for(i = 0; i < 3; i++)
		for(j = 0; j < 9; j++)
		{
			x = x_enemy - (4 - j) * 8;
			y = y_enemy + 8 * i;
			
			lcd_goto_xy_exact(0,0);
			
			if(enemymap[i][j] == 1)
			for(k = 0; k < 5; k++)
				for(l = 0; l < 5; l++)
					if(enemyA[k][l] == 1)
						lcd_setpixel(x + l,y + k);
		}
}

void collision()
{
	int i, j;
	
	if(y_proj <= y_enemy + 2 * 8)
	{
			for(j = 0; j < 3; j++)
			{
				for(i = 0; i < 9; i++)
				if(y_proj < y_enemy + 8 * j && y_proj > y_enemy + 8 * (j - 1))
				if(enemymap[j][i] != 0)
				if(x_proj > x_enemy - (4 - i) * 8 - 2 && x_proj < x_enemy - (4 - i) * 8 + 4)
				{
					enemymap[j][i] = 0;
					y_proj = 41;
					enemies_alive--;
				}
			}
	}
}


void move_enemies()
{
	if(x_e_step > 0 && x_e_counter == 20)
		x_enemy += 2 * x_e_step;
	if(x_e_step > 0 && x_e_counter > 40)
	{
		x_enemy += 2 * x_e_step;
		x_e_step *= -1;
	}
	
	if(x_e_step < 0 && x_e_counter == 20)
		x_enemy += 2 * x_e_step;
	if(x_e_step < 0 && x_e_counter < 0)
	{
		x_enemy += 2 * x_e_step;
		x_e_step *= -1;
	}
	
	x_e_counter += x_e_step;
}

void start_screen()
{
	char x = 0;
	lcd_clear();
	lcd_goto_xy_exact(0,0);
	lcd_goto_xy(4,1);
	lcd_chr(ARROW_LEFT);
	lcd_chr(ARROW_RIGHT);
	
	lcd_goto_xy(1,2);
	lcd_str("Butonul de jos pentru start");
	
	
	lcd_goto_xy(4,4);
	lcd_str("Enjoy!");
	
	_delay_ms(100);
	
	while(PINC & (1 << PC4)){
	_delay_ms(2000);
	}
}

void stop_screen()
{
	char x = 0;
	lcd_clear();
	lcd_goto_xy_exact(0,0);
	lcd_goto_xy(1,1);
	lcd_chr(ARROW_LEFT);
	printf(" Sfarsit ");
	lcd_chr(ARROW_RIGHT);
	
	lcd_goto_xy(1,2);
	lcd_str("Restart");
	
	
	lcd_goto_xy(4,4);
	lcd_str("Enjoy!");
	
	_delay_ms(100);
	while(PINC & (1 << PC4)){
	_delay_ms(2000);
	}
}





int main(void)
{	
	// Setup LCD
	lcd_init();
	lcd_contrast(0x40);
	
	initButtons();
	
	start_screen();
	
	
	initEnemies();
	while(1)
	{
		lcd_clear();
		
		/* check for collisions */
		collision();
		
		/* move and draw battleship*/
		move_ship();
		drawBattleship(x_ship,40);
		
		/* launch move and draw projectiles */
		launch_projectile();
		move_projectile();
		drawProjectile(x_proj, y_proj);
		
		/* draw enemies(only those who are alive) */
		drawEnemies();
		
		/* update lcd*/
		lcd_update();

		_delay_ms(300); 
		
		move_enemies();
		
		/* check if game ended */
		if(enemies_alive == 0)
		{
			stop_screen();
			initEnemies();
		}
	}
	
	
}
